from abaqus import *
from abaqusConstants import *
###
###
###
###
###
###
def bias_F(l1,L,B,n):
	return L*(1.-B**(1./(n-1.)))-l1*(1.-B**(n/(n-1.)))
###
###
###
def bias_dF(l1,L,B,n):
	return -L/(n-1.)*B**((2.-n)/(n-1))+l1*n/(n-1.)*B**(1./(n-1.))
###
###
###
def find_bias(l1,L,n,B0,tol=1e-6,imax=50):
	err=1.
	i=0
	print "(find_bias) l1 =",l1
	print "(find_bias) L =",L
	print "(find_bias) n =",n
	print "(find_bias) B0 =",B0
	success=True
	while err>tol:
		i+=1
		if i>imax:
			success=False
			break
		f0=bias_F(l1,L,B0,n)
		df0=bias_dF(l1,L,B0,n)
		B1=B0-f0/df0
		err=abs(B0-B1)
		B0=B1
	if B1==1.:
		success=False
	if success==True:
		print "(find_bias) i / imax =",i,"/",imax
		print "(find_bias) B1 =", B1, "err =",err
		return B1
	else:
		print "(find_bias) Error! Bias not found!"
###
###
###
###
###
###
### full triangular beam and support with chevron notch 
def achvbsT(

		jobBaseName="achvbsT",	# job name
		jobRootPath="./jobs",	# root folder for jobs
		jId=1,					# job Id
#
# beam geometry 
#
		Wc=1.0,					# beam width at center
		We=0.0,					# NOT USED
		B1=0.4,					# beam half thickness top
		B2=0.0,					# NOT USED
		S2=1.92,				# beam length up to point of applied force 
		a0=0.37,				# notch start depth
		a1=0.83,				# notch end depth
		a=0.4, 					# current crack length 
		Ss=8,					# support length
#
# material	
#
		matName="cubicSC",		# beam material: "cubicSC" or "genericLE"
		D11=166.,				# if material is "cubicSC" 
		D12=64.,				#   D11, D12 and D44 are elastic stiffness
		D44=80,					#   constants
		lcsX=(0.,0.,-1.),		#   lcsX and lcsXY define material orientation
		lcsXY=(0.,1.,0.),		# if material is "genericLE"
								#   D11 is Young's modulus
								#   D12 is Poisson's ratio
								#   and D44, lcsX and lcsXY are not used
#
# incrementation	
#
		minInc=0.1,				
		maxInc=0.5,
		initialInc=0.5,
		timePeriod=1,
		maxNumInc=10,
#
# loading 
#
		loadType="point",		# type of loading at beam free end
								#   can be: "point" or "line"
								# loading via prescribing displacement
		QQ1=0.,					# prescribed 1 displacement
		QQ2=0.01,				# prescribed 2 displacement
		QQ3=0.,					# prescribed 3 displacement
#
# fine-tuning mesh parameters
#
		tol_b=0.05,				# tolerance for smallness of crack front
								#   relative to notch root half length
								#   (see bmax bellow)
		no_b=5,					# no of segments of half crack front
								#   in case of crack front is small
		no_biasBh=5,			# control for initial biasBh sequence
		no_biasBv=4,			# control for initial biasBv sequence
		no_biasG=4,				# control for initial biasG sequence
	):
	
	import part
	import material
	import section
	import assembly
	import step
	import interaction
	import load
	import mesh
	import job
	import os
	import sys

	from numpy import ceil,floor,argmin,array,linspace

	loadConfig='canti'			# loading configuration is always canti
	
	b2=B1*(Wc-a1)/Wc			# notch root half length	
	b=b2*(a-a0)/(a1-a0) 		# crack front half length 

	bmax=b2*tol_b

# mesh element sizes 

	mnes=min(Wc/1e3,b/10.)		# min mesh element size
	ges=min(Wc/2.,1.0)			# global (maximal) mesh element size

	jobName=jobBaseName+"-"+str(jId)
	if jobName not in mdb.models.keys():
		myModel=mdb.Model(name=jobName)
	else: 
		myModel=mdb.models[jobBaseName]
	
###
###
### PART
###
###

###
### beam part
###	
	print "(achvbsT) beam part"

	myBeamPart=myModel.Part(name="beamPart",dimensionality=THREE_D,
			type=DEFORMABLE_BODY)
	beamSketch=myModel.ConstrainedSketch(name="beamSection",sheetSize=Wc)
	beamSketch.Line(point1=(0.,0.),point2=(B1,0.))
	if We!=0.0:
		beamSketch.Line(point1=(B1,0.),point2=(B1,We))
	beamSketch.Line(point1=(B1,We),point2=(B2,Wc))
	if B2!=0.0:
		beamSketch.Line(point1=(B2,Wc),point2=(0.,Wc))

	beamSketch.Line(point1=(0.,0.),point2=(-B1,0.))
	if We!=0.0:
		beamSketch.Line(point1=(-B1,0.),point2=(-B1,We))
	beamSketch.Line(point1=(-B1,We),point2=(-B2,Wc))
	if B2!=0.0:
		beamSketch.Line(point1=(-B2,Wc),point2=(0.,Wc))

	myBeamPart.BaseSolidExtrude(sketch=beamSketch,depth=S2)

# chevron sketch for partitioning cross-section
	
	bCHV=myModel.ConstrainedSketch(name="beamChevron",sheetSize=Wc,
			transform=(1,0,0,0,1,0,0,0,1,0,0,S2))
	bCHV.Line(point1=(0,a),point2=(b,a))
	bCHV.Line(point1=(b,a),point2=(b2,a1))
	bCHV.Line(point1=(0,a),point2=(-b,a))
	bCHV.Line(point1=(-b,a),point2=(-b2,a1))
	bCHV.Line(point1=(0,0),point2=(0,Wc))

	print "(achvbsT) partitioning beam face with beamChevron"

	myBeamPart.PartitionFaceBySketch(
		faces=myBeamPart.faces.findAt(((0.5*b,a,S2),)),
		sketch=bCHV,
		sketchOrientation=LEFT)
	
# beam geometry sets 

	myBeamPart.Set(name="BEAM",cells=myBeamPart.cells.findAt(((0.,0.,0.),),))

	myBeamPart.Set(name="NOTCH",
		faces=myBeamPart.faces.findAt(
		((-0.5*b,1.01*a,S2),),((-0.5*b,0.99*a,S2),),
		((0.5*b,1.01*a,S2),),((0.5*b,0.99*a,S2),)))

	myBeamPart.Set(name="END_FACE",
		faces=myBeamPart.faces.findAt(
		((-0.5*b,1.01*a,0),),((-0.5*b,0.99*a,0),),
		((0.5*b,1.01*a,0),),((0.5*b,0.99*a,0),)))

	myBeamPart.Set(name="CHV",
		faces=myBeamPart.faces.findAt(
			((-b,1.01*a,S2),),((b,1.01*a,S2),)))

	myBeamPart.Set(name="CRACK_FRONT",
		edges=myBeamPart.edges.findAt(
			((-0.5*b,a,S2),),((0.5*b,a,S2),),))

	myBeamPart.Set(name="SIF_LINE",
		edges=myBeamPart.edges.findAt(
			((0,(a+Wc)/2.,S2),),))

# beam cross-section partitioning
# blue horizontal region 

	l=sqrt(b2**2+(Wc-a1)**2)
	k=sqrt((B1-b2)**2+a1**2)

# biased element size along the crack front 
# is switched on for larger front
# see parameter bmax

	if b>bmax:
		print "(achvbsT) Biasing elements of crack front is ON!"

		biasBh=[0]
		for i in range(no_biasBh):
			biasBh.append(mnes)

		while sum(biasBh)<b:
			biasBh=biasBh+[biasBh[-1]+biasBh[-2]]
		biasBh=biasBh[0:-1]
		biasBh[-1]=b-0.5*(sum(biasBh[0:-1])+b)
	else:

# for small crack-front length, 
# crack front is divided into 5 equal parts

		print "(achvbsT) Biasing elements of crack front is OFF!"
		mnes=b/no_b

		biasBh=[0]
		for i in range(no_biasBh):
			biasBh.append(mnes)

		while sum(biasBh)<b:
			biasBh=biasBh+[biasBh[-1]]

# blue vertical region

	biasBv=[0]
	for i in range(no_biasBv):
		biasBv.append(mnes)

	while sum(biasBv)<a:
		biasBv=biasBv+[biasBv[-1]+biasBv[-2]]
	biasBv=biasBv[0:-1]
	biasBv[-1]=a-0.5*(sum(biasBv[0:-1])+a)

# green bias region 

	m=sqrt((b2-b)**2+(a1-a)**2)

	biasG=[0]
	for i in range(no_biasG):
		biasG.append(mnes)

	while sum(biasG)<m:
		if (biasG[-1]+biasG[-2])<ges:
			biasG=biasG+[biasG[-1]+biasG[-2]]
		else:
			biasG=biasG+[10*mnes]
	biasG=biasG[0:-1]
	biasG[-1]=m-0.5*(sum(biasG[0:-1])+m)

# make cross-section sketches
# record x,y coordinates of nodes at crack front line in cross-section sketch 

	bCSM=myModel.ConstrainedSketch(name="beamCrossSectionMesh",sheetSize=Wc,
			transform=(1,0,0,0,1,0,0,0,1,0,0,S2))

	crackFrontPts=[(0,a)]
		
	qnXps=[(0,0)]  				# X-coord of quarter nodes above crack front
	qnYp=a+0.5*mnes*(a1-a)/m	# Y-coord of quarter nodes above crack front

	db=0.0
	for i in range(len(biasBh)):
		db=db+biasBh[i]
		x=b-db 
		dl=db*l/b
		db1=dl*B1/(l+k)
		da1=dl*Wc/(l+k)
		x1=b2-db1
		y1=a1+da1
		bCSM.Line(point1=(x,0),point2=(x,a))
		bCSM.Line(point1=(x,a),point2=(x1,y1))
		crackFrontPts.append((x,a))
		qnXps.append([
			(x+0.5*mnes*((a1-a)/m)*(x1-x)/(y1-a)),
			0.5*x+0.5*(x+0.5*mnes*((a1-a)/m)*(x1-x)/(y1-a))
		])
		bCSM.Line(point1=(-x,0),point2=(-x,a))
		bCSM.Line(point1=(-x,a),point2=(-x1,y1))
		crackFrontPts.append((-x,a))
		qnXps.append([
			-(x+0.5*mnes*((a1-a)/m)*(x1-x)/(y1-a)),
			-(0.5*x+0.5*(x+0.5*mnes*((a1-a)/m)*(x1-x)/(y1-a)))
		])

	db=0.0
	for i in range(len(biasBv)):
		db=db+biasBv[i]
		y=a-db
		dk=db*k/a
		db2=dk*B1/(l+k)
		da2=dk*Wc/(l+k)
		x2=b2+db2
		y2=a1-da2
		bCSM.Line(point1=(0,y),point2=(b,y))
		bCSM.Line(point1=(b,y),point2=(x2,y2))
		bCSM.Line(point1=(0,y),point2=(-b,y))
		bCSM.Line(point1=(-b,y),point2=(-x2,y2))

	dm=0.0
	for i in range(len(biasG)):
		dm=dm+biasG[i]
		db=dm*(b2-b)/m
		da=dm*(a1-a)/m
		y=a+da
		x=b+db

# mesh zone of parallel lines

		if i<no_biasG+1:		
			bCSM.Line(point1=(0.,y),point2=(x,y))
			bCSM.Line(point1=(x,y),point2=(x,0.))

			bCSM.Line(point1=(0.,y),point2=(-x,y))
			bCSM.Line(point1=(-x,y),point2=(-x,0.))

		else:	

# biased mesh above zone of parallel lines

			mp=sum(biasG[0:no_biasG+1])
			bp=mp*(b2-b)/m
			ap=mp*(a1-a)/m
			dmc=dm-mp
			da1=dmc*(Wc-a-ap)/(m-mp)
			db2=dmc*(B1-b-bp)/(m-mp)
			y1=a+ap+da1
			x2=b+bp+db2
			bCSM.Line(point1=(0,y1),point2=(x,y))
			bCSM.Line(point1=(x,y),point2=(x2,0.))
			bCSM.Line(point1=(0,y1),point2=(-x,y))
			bCSM.Line(point1=(-x,y),point2=(-x2,0.))

	print "(achvbsT) partitioning beam face with beamCrossSectionMesh"

	myBeamPart.PartitionFaceBySketchThruAll(
		faces=myBeamPart.faces.findAt(
			((b,1.01*a,S2),),
			((b,0.99*a,S2),),
			((b,a,0),),
			((-b,1.01*a,S2),),
			((-b,0.99*a,S2),),
			((-b,a,0),)),
		sketchPlane=myBeamPart.faces.findAt((b,1.01*a,S2)),
		sketchPlaneSide=SIDE1,
		sketch=bCSM,
		sketchUpEdge=myBeamPart.edges.findAt((0.0,0.5*Wc,S2)),
		sketchOrientation=LEFT)

# beam part meshing

	myBeamElement=mesh.ElemType(elemCode=C3D20,elemLibrary=STANDARD)
	
	print "(achvbsT) beam part element type and mesh controls"

	for cell in myBeamPart.cells:
		myBeamPart.setElementType(regions=(cell,),elemTypes=(myBeamElement,))
		myBeamPart.setMeshControls(regions=(cell,),
			elemShape=HEX,technique=BOTTOM_UP)

	print "(achvbsT) general seed beam part"

	myBeamPart.seedPart(size=ges)		

	print "(achvbsT) generate beam part mesh"

	nelS2=20
	biasS2=int(floor(ges/mnes))
	myBeamPart.generateBottomUpExtrudedMesh(
		cell=myBeamPart.cells[0],
		numberOfLayers=nelS2,
		extrudeVector=((0,0,S2),(0,0,0)),
		geometrySourceSide=myBeamPart.sets["NOTCH"],
		biasRatio=biasS2
	)

# beam BC node sets

	print "(achvbsT) setting beam BC node sets"

# beam load node set	

	beamLoadNodeLabels=[]

	if loadType=="point":	
		ns=myBeamPart.nodes.getByBoundingSphere(
			center=(0.,0.,0.,),radius=mnes/5.
		)	
		beamLoadNodeLabels.append(ns[0].label)

	if loadType=="line":
		ns=myBeamPart.nodes.getByBoundingCilinder(
			center1=(0.-mnes,0.,0.,),center2=(B1+mnes,0.,0.,),radius=mnes/5.
		)

	for n in ns:	
		beamLoadNodeLabels.append(n.label)
	myBeamPart.SetFromNodeLabels(name="LOAD",nodeLabels=beamLoadNodeLabels)

# edge-point displacement

	ns=myBeamPart.nodes.getByBoundingSphere(
		center=(B1,0.,0.,),radius=mnes/5.
	)	
	myBeamPart.SetFromNodeLabels(name="EDGEPOINT",nodeLabels=[ns[0].label])


# setting mid nodes in quarter point elements at crack front

	print "(achvbsT) setting beam quarter point elements"

# quarter nodes for Y below the crack front
# quarter nodes with Z below S2

	i=0
	lS2=S2*(1.-biasS2**(1./(nelS2-1.)))/(1.-biasS2**(nelS2/(nelS2-1.)))
	for x,y in crackFrontPts:
		if abs(x)<=b:
			n=myBeamPart.nodes.getByBoundingSphere(
				center=(x,y-0.5*mnes,S2),radius=0.1*mnes)
			myBeamPart.editNode(nodes=n,
				coordinate1=x,coordinate2=y-0.25*mnes,coordinate3=S2)
			n=myBeamPart.nodes.getByBoundingSphere(
				center=(x,y,S2-0.5*lS2),radius=0.1*lS2)
			myBeamPart.editNode(nodes=n,
				coordinate1=x,coordinate2=y,coordinate3=S2-0.25*lS2)
			i=i+2

# quarter nodes for Y above crack front	

	for qnXpOld,qnXpNew in qnXps:
		n=myBeamPart.nodes.getByBoundingSphere(
			center=(qnXpOld,qnYp,S2),radius=0.1*mnes)
		myBeamPart.editNode(nodes=n,
			coordinate1=qnXpNew,coordinate2=0.5*(a+qnYp),coordinate3=S2)
		i=i+1

	print "(achvbsT) beam part no. quarter nodes", i+1

# local beam CS

	print "(achvbsT) defining beam part material orientation"

	if matName=="cubicSC":
		bX=myBeamPart.DatumPointByCoordinate(coords=lcsX)	
		bXY=myBeamPart.DatumPointByCoordinate(coords=lcsXY)
		bCsys=myBeamPart.DatumCsysByThreePoints(
			coordSysType=CARTESIAN,
			origin=myBeamPart.vertices.findAt((0.,0.,0.)),
			point1=myBeamPart.datums[bX.id],
			point2=myBeamPart.datums[bXY.id]
		)
		myBeamPart.MaterialOrientation(
			region=myBeamPart.sets["BEAM"],
			localCsys=myBeamPart.datums[bCsys.id],
			orientationType=SYSTEM,
		)

###
### support part
###
	
	print "(achvbsT) support part"
	
	mySuppPart=myModel.Part(name="suppPart",dimensionality=THREE_D,
			type=DEFORMABLE_BODY)

	mySuppPart.BaseSolidExtrude(sketch=beamSketch,depth=Ss)

# chevron face on supportPart

	sCHV=myModel.ConstrainedSketch(
		name="suppChevron",
		objectToCopy=bCHV
	)
	
	print "(achvbsT) partitioning support part with suppChevron"

	mySuppPart.PartitionFaceBySketch(
		sketch=sCHV,
		faces=mySuppPart.faces.findAt(((0.5*B2,0.5*Wc,0),)),
		sketchOrientation=LEFT
	)

# chevron support part sets

	mySuppPart.Set(name="SUPPORT",
		cells=mySuppPart.cells.findAt(((0.,0.,0.),),))

	mySuppPart.Set(name="CHV",
		faces=mySuppPart.faces.findAt(
			((-b,1.01*a,0),),((b,1.01*a,0),))
	)	

	mySuppPart.Set(name="NOTCH",
		faces=mySuppPart.faces.findAt(
			((-0.5*b,1.01*a,0),),((-0.5*b,0.99*a,0),),
			((0.5*b,1.01*a,0),),((0.5*b,0.99*a,0),))
	)

	mySuppPart.Set(name="CRACK_FRONT",
		edges=mySuppPart.edges.findAt(
		((-0.5*b,a,0),),((0.5*b,a,0),),)
	)

# support part chevron face is the same as for the beam
# mesh sketch for support part chevron face and ...

	sCSM=myModel.ConstrainedSketch(
		name="suppChevronMesh",
		objectToCopy=bCSM
	)

	print "(achvbsT) partitioning support part with suppChevronMesh"

	mySuppPart.PartitionFaceBySketch(
		faces=mySuppPart.faces.findAt(
			((-b,1.01*a,0),),
			((-b,0.99*a,0),),
			((b,1.01*a,0),),
			((b,0.99*a,0),)),
		sketchOrientation=LEFT,
		sketch=sCSM
	)

# support part mesh

	print "(achvbsT) support part element type and mesh control"

	for cell in mySuppPart.cells:
		mySuppPart.setElementType(regions=(cell,),elemTypes=(myBeamElement,))
		mySuppPart.setMeshControls(regions=(cell,),
			elemShape=HEX,technique=BOTTOM_UP)

	print "(achvbsT) general seed support part"

	mySuppPart.seedPart(size=ges)		

	print "(achvbsT) generate support part mesh"
	
	nelSs=int(nelS2/2)
	l1S2=S2*(1.0-biasS2**(1./(nelS2-1.0)))/(1.0-biasS2**(nelS2/(nelS2-1.)))
	biasSs=find_bias(l1S2,Ss,nelSs,biasS2)

	mySuppPart.generateBottomUpExtrudedMesh(
		cell=mySuppPart.cells[0],
		numberOfLayers=nelSs,
		extrudeVector=((0,0,0),(0,0,Ss)),
		geometrySourceSide=mySuppPart.sets["NOTCH"],
		biasRatio=biasSs
	)

# BC support node sets

	print "(achvbsT) setting support BC node sets"

# support end node set
	suppEndNodeLabels=[]
	ns=mySuppPart.nodes.getByBoundingBox(
		xMin=-(B1+mnes),yMin=0.-mnes,zMin=Ss-mnes/5.,
		xMax=B1+mnes,yMax=Wc+mnes,zMax=Ss+mnes
	)
	for n in ns:
		suppEndNodeLabels.append(n.label)
	mySuppPart.SetFromNodeLabels(name="END",nodeLabels=suppEndNodeLabels)

# setting mid nodes in quarter point elements at crack front

	print "(achvbsT) setting support quarter point elements"

# quarter nodes for Y below the crack front
# quarter nodes with Z below S2

	i=0
	lSs=Ss*(1.-biasSs**(1./(nelSs-1.)))/(1-biasSs**(nelSs/(nelSs-1.)))
	for x,y in crackFrontPts:
		if abs(x)<=b:
			n=mySuppPart.nodes.getByBoundingSphere(
				center=(x,y-0.5*mnes,0.),radius=0.1*mnes)
			mySuppPart.editNode(nodes=n,
				coordinate1=x,coordinate2=y-0.25*mnes,coordinate3=0.)
			n=mySuppPart.nodes.getByBoundingSphere(
				center=(x,y,0.5*lSs),radius=0.1*lSs)
			mySuppPart.editNode(nodes=n,
				coordinate1=x,coordinate2=y,coordinate3=0.25*lSs)
			i=i+2

# quarter nodes for Y above crack front	

	for qnXpOld,qnXpNew in qnXps:	
		n=mySuppPart.nodes.getByBoundingSphere(
			center=(qnXpOld,qnYp,0.),radius=0.1*mnes)
		mySuppPart.editNode(nodes=n,
			coordinate1=qnXpNew,coordinate2=0.5*(a+qnYp),coordinate3=0.)
		i=i+1

	print "(achvbsT) support part no. quarter nodes", i+1

# local support CS

	print "(achvbsT) defining support part material orientation"

	if matName=="cubicSC":
		sX=mySuppPart.DatumPointByCoordinate(coords=lcsX)	
		sXY=mySuppPart.DatumPointByCoordinate(coords=lcsXY)
		sCsys=mySuppPart.DatumCsysByThreePoints(
			coordSysType=CARTESIAN,
			origin=mySuppPart.vertices.findAt((0.,0.,0.)),
			point1=mySuppPart.datums[sX.id],
			point2=mySuppPart.datums[sXY.id]
		)
		mySuppPart.MaterialOrientation(
			region=mySuppPart.sets["SUPPORT"],
			localCsys=mySuppPart.datums[sCsys.id],
			orientationType=SYSTEM,
		)


####
####
#### MATERIAL
####
####
	
	myMaterial=myModel.Material(name=matName)

	if matName=="genericLE":
		myMaterial.Elastic(type=ISOTROPIC,table=((D11,D12),))

	if matName=="cubicSC":
		myMaterial.Elastic(type=ORTHOTROPIC,
		table=((D11,D12,D11,D12,D12,D11,D44,D44,D44),))

####
####
#### SECTION	
####
####
	
	myBeamSection=myModel.HomogeneousSolidSection(name="beamSection",
		material=matName)
	myBeamPart.SectionAssignment(region=(myBeamPart.cells,),
		sectionName="beamSection")

	mySuppSection=myModel.HomogeneousSolidSection(name="suppSection",
		material=matName)
	mySuppPart.SectionAssignment(region=(mySuppPart.cells,),
		sectionName="suppSection")

	
####
####
#### ASSEMBLY
####
####
	
	myAssembly=myModel.rootAssembly
	
	myBeamInstance=myAssembly.Instance(name="beamInstance",part=myBeamPart,
		dependent=ON)
	mySuppInstance=myAssembly.Instance(name="suppInstance",part=mySuppPart,
		dependent=ON)
	mySuppInstance.translate(vector=(0,0,S2))	

# reference point set for prescribed BC

	if loadConfig=='canti':	
		rpQQ_xyz=(0.,0.,0.)
	rpQQ=myAssembly.ReferencePoint(point=rpQQ_xyz)
	rpQQ_key=myAssembly.referencePoints.keys()[0]
	sQQ=myAssembly.Set(name="QQ",
		referencePoints=(myAssembly.referencePoints[rpQQ_key],))
	

####
####
#### STEP
####
####
	
	myModel.StaticStep(name="statics0",previous="Initial",
		timePeriod=timePeriod, initialInc=initialInc,
		minInc=minInc,maxInc=maxInc,maxNumInc=maxNumInc,nlgeom=True)

	myModel.fieldOutputRequests["F-Output-1"].setValues(
		variables=("S","U"))

	myModel.FieldOutputRequest(
		name="SIF_LINE", createStepName='statics0',
		region=myBeamInstance.sets["SIF_LINE"],
		variables=("S",),)

	myModel.historyOutputRequests["H-Output-1"].setValues(
		region=sQQ,variables=("RF1","RF2","RF3","U1","U2","U3"))

	myModel.HistoryOutputRequest(
		name="EdgePointDisplacement",createStepName='statics0',
		region=myBeamInstance.sets["EDGEPOINT"], 
		variables=('U1',"U2","U3"),
	)

	
####
####
#### INTERACTIONS
####
####
	if loadConfig=="canti":	
		myModel.Equation(name="QQ_END",
			terms=((1.,"beamInstance.LOAD",2),(-1.,"QQ",2)))

	myModel.Tie(name="CHEVRON",
		master=mySuppInstance.sets["CHV"],slave=myBeamInstance.sets["CHV"],
		constraintEnforcement=NODE_TO_SURFACE	

	)
	
####
####
#### BC
####
####

	if loadConfig=="canti":
		myModel.DisplacementBC(name="fixed",createStepName="statics0",
			region=mySuppInstance.sets["END"],u1=0,u2=0,u3=0)
	
# prescribed deformation
	myModel.DisplacementBC(name="QQ",createStepName="statics0",
		region=sQQ,u1=QQ1,u2=QQ2,u3=QQ3)

	
####
####
#### JOB
####
####
	
	jobPath=jobRootPath+"/"+jobName
	myJob=mdb.Job(name=jobName,model=jobName)
	
	sys.__stderr__.write("\n(achvbsT) Setting file ... "+jobName+" ... ")
	cwd=os.getcwd()
	if os.path.exists(jobPath)==False:
		os.makedirs(jobPath)
		os.chdir(jobPath)
		myJob.writeInput()
		os.chdir(cwd)
		sys.__stderr__.write("OK!\n")
	else:
		sys.__stderr__.write(
			"\n(achvbsT) .inp file exists ...SKIPPED \n")

