# libabaqus_python.py
# library of various abaqus python scripts 

from abaqusConstants import *

###
###
###
def reportHistoryRequestParameter(odbFile,parameter="",setType="",
	stepName="",instanceName="",setName=""):
# works only for nodal values of U and RF	

	import odbAccess
	import os
	from numpy import zeros

	jobPath,jobOdbName=os.path.split(odbFile)
	jobName=jobOdbName.rstrip(".odb")
	jobDataPath="data/"+jobName

	myOdb=odbAccess.openOdb(path=odbFile)
	myStep=myOdb.steps[stepName]

	nodeData={}

	if setType=="root":
		nSet=myOdb.rootAssembly.nodeSets[setName].nodes[0]
	if setType=="part":
		nSet=myOdb.rootAssembly.instances[instanceName].nodeSets[setName].nodes
	for n in nSet:
		nKey="Node "+instanceName+"."+str(n.label)
		nData=myStep.historyRegions[nKey].historyOutputs[parameter].data
		nodeData[n.label]=nData

	fileName=jobDataPath+"/"+jobName+"_"+setName+"-"+stepName+"_"+parameter+".hro"
	print "(reportHistoryRequestParameter) Writing ...", fileName
	if os.path.isdir(jobDataPath)==False:
		os.makedirs(jobDataPath)
	f=open(fileName,"w")

	f.write("# Data retrieved by reportHistoryRequestParameter.\n")
	f.write("# History output for parameter "+parameter+" of nodes in "+setName+".\n")
	f.write("# Displacements are in 1e-6m, and forces in 1e-3N.\n")
	f.write("# Data format per line: nodeID, value_frame-1, value_frame-2, ... , value-last-frame\n")
	
	for nKey in nodeData.keys():
		line="% 5s " % str(nKey)
		for d in nodeData[nKey]:
			line=line+"% 13s " % (str(d[1]))
		f.write(line+"\n")
	f.close()
###
###
###
def reportComponentFromFieldRequest(odbFile, field="S", fComp=1, 
	instanceName="",stepName="",setName="", setType="", frames=[-1]):
	"""
	The nodal field component (stress, strain) of nodes in set from Field Requests.
	Nodal values are averaged over elements that share the node of a set.
	""" 
	
	import odbAccess
	import os 

	jobPath,jobOdbName=os.path.split(odbFile)
	jobName=jobOdbName.rstrip(".odb")
	jobDataPath="data/"+jobName+"/"

	myOdb=odbAccess.openOdb(path=odbFile)
	myStep=myOdb.steps[stepName]

	if frames==[]:
		frames=range(1,len(myStep.frames))
		print "(reportComponentFromFieldRequest) Processing ALL frames!"

	if os.path.isdir(jobDataPath)==False:
		os.makedirs(jobDataPath)
	fileName=jobDataPath+jobName+"_"+setName+"-"+stepName+"_"+field+str(fComp)+".fro"
	f=open(fileName,"w")
	print "(reportComponentFromFieldRequest) Writing file ... ", fileName
	f.write("# File automatically generated by reportComponentFromFieldRequest.\n")
	f.write("# Nodal field component "+field+str(fComp)+" for set name "+setName+".\n")
	f.write("# Stress components are units of 1e9Pa.\n")
	f.write("# Data format per line:\n")
	f.write("# % 6s % 8s % 13s % 13s % 13s % 13s\n" % 
		("frame", "node", "X coord", "Y coord", "Z coord", field+str(fComp)))
	for frame in frames:
		print "(reportComponentFromFieldRequest) \tframe ... ", frame
		myFrame=myStep.frames[frame]
		myRegions=myStep.historyRegions
		
		F=myFrame.fieldOutputs[field]
		F_nodal=F.getSubset(
			region=myOdb.rootAssembly.elementSets[" ALL ELEMENTS"],
			position=ELEMENT_NODAL)

		if setType=="root":
			nodes=myOdb.rootAssembly.nodeSets[setName].nodes[0]
			F_set=F_nodal.getSubset(
				region=myOdb.rootAssembly.nodeSets[setName])

		if setType=="part":
			nodes=myOdb.rootAssembly.instances[
				instanceName].nodeSets[setName].nodes
			F_set=F_nodal.getSubset(
				region=myOdb.rootAssembly.instances[
					instanceName].nodeSets[setName])

		FC={}
		for n in nodes:
			sumFC=0.
			count=0.
			for v in F_set.values:
				if v.nodeLabel==n.label:
					count+=1.
					sumFC+=v.data[fComp-1]
			FC[n.label]=sumFC/count
			f.write("% 8i % 8i % 12e % 12e % 12e % 12e\n" %
				(frame,n.label,n.coordinates[0],n.coordinates[1],
				n.coordinates[2],FC[n.label]))
	f.close()



