from numpy import linspace,cos,sin,pi,sqrt,array,dot,arccos

# default unit system for parameters is:
# 	um for length
#	mN for forces
#	GPa=mN/um^2 for stress

jobBaseName="testCHV2"

# prescribed displacements

QQ2=0.01

# geometry definition
# for parameter definitions see libabaqus_achvbsT.py

Wc=2.86  ##
B1=1.16  ##
S2=11.64 ##
a0=0.57  ##
a1=1.59  ##
a=0.58   ##
Ss=1.34  ##

# material parameters
# example for single crystal Si

matName="cubicSC"
D11=166.
D12=64.
D44=80.

# code to define material orientation

# material orientation
# beam axis (Z) is [111]
# up-direction (Y) is [-110]

# global CS
u0=array([1.,0.,0.,])
v0=array([0.,1.,0.,])
w0=array([0.,0.,1.,])

def Rz(a,v):
	ca=cos(a)
	sa=sin(a)
	R=array([
		[ca, -sa, 0],
		[sa,  ca, 0],
		[0.,  0., 1.]
	])
	return dot(R,v)

def Ry(a,v):
	ca=cos(a)
	sa=sin(a)
	R=array([
		[ ca, 0., sa],
		[ 0., 1., 0.],
		[-sa, 0., ca]
	])
	return dot(R,v)

u1=Rz(-pi/4., u0)
v1=Rz(-pi/4., v0)
w1=Rz(-pi/4., w0)

u2=Ry(-arccos(sqrt(1./3.)), u1)
v2=Ry(-arccos(sqrt(1./3.)), v1)
w2=Ry(-arccos(sqrt(1./3.)), w1)

#print u0,v0,w0
#print u1,v1,w1
#print u2,v2,w2

lcsX=u2
lcsXY=v2

# defining crack length vector 

nJobs=2
crack_lengths=a0+(a1-a0)*linspace(0.05,0.90,nJobs)


# main 
if __name__=="__main__":

	from libabaqus_achvbsT import achvbsT

# individual test run
# for calculating the initial beam compliance
# use small crack length a

#	achvbsT(
#		jobBaseName=jobBaseName,jId=1,
#		matName=matName,
#		D11=D11,D12=D12,D44=D44,
#		lcsX=lcsX,lcsXY=lcsXY,
#		Wc=Wc,B1=B1,S2=S2,
#		a0=a0,a1=a1,a=a,
#		Ss=Ss,
#		QQ2=QQ2,
#	)

# generation of compliance vs. crack length data
# loop over crack lengths and generate abaqus input files

	i=1
	for a in crack_lengths:
		achvbsT(
			jobBaseName=jobBaseName,jId=i,
			matName=matName,
			D11=D11,D12=D12,D44=D44,
			lcsX=lcsX,lcsXY=lcsXY,
			Wc=Wc,B1=B1,S2=S2,
			a0=a0,a1=a1,a=a,
			Ss=Ss,
			QQ2=QQ2
		)
		i+=1




